@@ -1,6 +1,6 @@
 #-------------------------------------------------------------------------------------------------
 # Pure Perl interface of Tokyo Tyrant
-#                                                       Copyright (C) 2006-2008 Mikio Hirabayashi
+#                                                                Copyright (C) 2006-2010 FAL Labs
 # This file is part of Tokyo Tyrant.
 # Tokyo Tyrant is free software; you can redistribute it and/or modify it under the terms of
 # the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -23,7 +23,7 @@ use Carp;
 
 require Exporter;
 use base qw(Exporter);
-our $VERSION = '1.15';
+our $VERSION = '1.16';
 our $DEBUG = 0;
 
 
@@ -117,6 +117,7 @@ sub new {
     my $self = {
         _ecode => ESUCCESS,
         _sock => undef,
+        _tout => undef,
     };
     bless($self, $class);
     return $self;
@@ -175,6 +176,7 @@ sub open {
     my $self = shift;
     my $host = shift;
     my $port = shift;
+    my $timeout = shift;
     if(scalar(@_) != 0 || !defined($host) || length($host) < 1){
         croak((caller(0))[3] . ": invalid parameter") if($TokyoTyrant::DEBUG);
         return undef;
@@ -195,12 +197,12 @@ sub open {
             $self->{_ecode} = EREFUSED;
             return 0;
         }
+        setsockopt($sock, IPPROTO_TCP, TCP_NODELAY, 1);
         if(!connect($sock, $sin)){
             $self->{_ecode} = EREFUSED;
             close($sock);
             return 0;
         }
-        setsockopt($sock, IPPROTO_TCP, TCP_NODELAY, 1);
     } else {
         my $sun = sockaddr_un($host);
         if(!socket($sock, PF_UNIX, SOCK_STREAM, 0)){
@@ -214,6 +216,7 @@ sub open {
         }
     }
     $self->{_sock} = $sock;
+    $self->{_tout} = defined($timeout) && $timeout > 0 ? $timeout : 0;
     return 1;
 }
 
@@ -254,11 +257,11 @@ sub put {
     }
     my $sbuf = pack("CCNN", 0xC8, 0x10, bytes::length($key), bytes::length($value));
     $sbuf .= $key . $value;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -286,11 +289,11 @@ sub putkeep {
     }
     my $sbuf = pack("CCNN", 0xC8, 0x11, bytes::length($key), bytes::length($value));
     $sbuf .= $key . $value;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -318,11 +321,11 @@ sub putcat {
     }
     my $sbuf = pack("CCNN", 0xC8, 0x12, bytes::length($key), bytes::length($value));
     $sbuf .= $key . $value;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -353,11 +356,11 @@ sub putshl {
     }
     my $sbuf = pack("CCNNN", 0xC8, 0x13, bytes::length($key), bytes::length($value), $width);
     $sbuf .= $key . $value;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -385,7 +388,7 @@ sub putnr {
     }
     my $sbuf = pack("CCNN", 0xC8, 0x18, bytes::length($key), bytes::length($value));
     $sbuf .= $key . $value;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
@@ -407,11 +410,11 @@ sub out {
     }
     my $sbuf = pack("CCN", 0xC8, 0x20, bytes::length($key));
     $sbuf .= $key;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -438,11 +441,11 @@ sub get {
     }
     my $sbuf = pack("CCN", 0xC8, 0x30, bytes::length($key));
     $sbuf .= $key;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return undef;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return undef;
@@ -451,12 +454,12 @@ sub get {
         $self->{_ecode} = ENOREC;
         return undef;
     }
-    my $vsiz = _recvint32($sock);
+    my $vsiz = $self->_recvint32();
     if($vsiz < 0){
         $self->{_ecode} = ERECV;
         return undef;
     }
-    my $vref = _recv($sock, $vsiz);
+    my $vref = $self->_recv($vsiz);
     if(!defined($vref)){
         $self->{_ecode} = ERECV;
         return undef;
@@ -484,12 +487,12 @@ sub mget {
         $rnum++;
     }
     $sbuf = pack("CCN", 0xC8, 0x31, $rnum) . $sbuf;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return -1;
     }
-    my $code = _recvcode($sock);
-    $rnum = _recvint32($sock);
+    my $code = $self->_recvcode();
+    $rnum = $self->_recvint32();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return -1;
@@ -504,14 +507,14 @@ sub mget {
     }
     %$recs = ();
     for(my $i = 0; $i < $rnum; $i++){
-        my $ksiz = _recvint32($sock);
-        my $vsiz = _recvint32($sock);
+        my $ksiz = $self->_recvint32();
+        my $vsiz = $self->_recvint32();
         if($ksiz < 0 || $vsiz < 0){
             $self->{_ecode} = ERECV;
             return -1;
         }
-        my $kref = _recv($sock, $ksiz);
-        my $vref = _recv($sock, $vsiz);
+        my $kref = $self->_recv($ksiz);
+        my $vref = $self->_recv($vsiz);
         if(!defined($kref) || !defined($vref)){
             $self->{_ecode} = ERECV;
             return -1;
@@ -536,11 +539,11 @@ sub vsiz {
     }
     my $sbuf = pack("CCN", 0xC8, 0x38, bytes::length($key));
     $sbuf .= $key;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return -1;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return -1;
@@ -549,7 +552,7 @@ sub vsiz {
         $self->{_ecode} = ENOREC;
         return -1;
     }
-    return _recvint32($sock);
+    return $self->_recvint32();
 }
 
 
@@ -565,11 +568,11 @@ sub iterinit {
         return 0;
     }
     my $sbuf = pack("CC", 0xC8, 0x50);
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -594,11 +597,11 @@ sub iternext {
         return undef;
     }
     my $sbuf = pack("CC", 0xC8, 0x51);
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return undef;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return undef;
@@ -607,12 +610,12 @@ sub iternext {
         $self->{_ecode} = ENOREC;
         return undef;
     }
-    my $ksiz = _recvint32($sock);
+    my $ksiz = $self->_recvint32();
     if($ksiz < 0){
         $self->{_ecode} = ERECV;
         return undef;
     }
-    my $kref = _recv($sock, $ksiz);
+    my $kref = $self->_recv($ksiz);
     if(!defined($kref)){
         $self->{_ecode} = ERECV;
         return undef;
@@ -640,11 +643,11 @@ sub fwmkeys {
     }
     my $sbuf = pack("CCNN", 0xC8, 0x58, bytes::length($prefix), $max);
     $sbuf .= $prefix;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return \@keys;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return \@keys;
@@ -653,18 +656,18 @@ sub fwmkeys {
         $self->{_ecode} = ENOREC;
         return \@keys;
     }
-    my $knum = _recvint32($sock);
+    my $knum = $self->_recvint32();
     if($knum < 0){
         $self->{_ecode} = ERECV;
         return \@keys;
     }
     for(my $i = 0; $i < $knum; $i++){
-        my $ksiz = _recvint32($sock);
+        my $ksiz = $self->_recvint32();
         if($ksiz < 0){
             $self->{_ecode} = ERECV;
             return \@keys;
         }
-        my $kref = _recv($sock, $ksiz);
+        my $kref = $self->_recv($ksiz);
         if(!defined($kref)){
             $self->{_ecode} = ERECV;
             return \@keys;
@@ -691,11 +694,11 @@ sub addint {
     }
     my $sbuf = pack("CCNN", 0xC8, 0x60, bytes::length($key), $num);
     $sbuf .= $key;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return undef;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return undef;
@@ -704,7 +707,7 @@ sub addint {
         $self->{_ecode} = EKEEP;
         return undef;
     }
-    return _recvint32($sock);
+    return $self->_recvint32();
 }
 
 
@@ -727,11 +730,11 @@ sub adddouble {
     my $sbuf = pack("CCN", 0xC8, 0x61, bytes::length($key));
     $sbuf .= _packquad($integ) . _packquad($fract);
     $sbuf .= $key;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return undef;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return undef;
@@ -740,8 +743,8 @@ sub adddouble {
         $self->{_ecode} = EKEEP;
         return undef;
     }
-    $integ = _recvint64($sock);
-    $fract = _recvint64($sock);
+    $integ = $self->_recvint64();
+    $fract = $self->_recvint64();
     return $integ + $fract / 1000000000000;
 }
 
@@ -768,11 +771,11 @@ sub ext {
     my $sbuf = pack("CCNNNN", 0xC8, 0x68,
                     bytes::length($name), $xopts, bytes::length($key), bytes::length($value));
     $sbuf .= $name . $key . $value;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return undef;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return undef;
@@ -781,12 +784,12 @@ sub ext {
         $self->{_ecode} = EMISC;
         return undef;
     }
-    my $xsiz = _recvint32($sock);
+    my $xsiz = $self->_recvint32();
     if($xsiz < 0){
         $self->{_ecode} = ERECV;
         return undef;
     }
-    my $xref = _recv($sock, $xsiz);
+    my $xref = $self->_recv($xsiz);
     if(!defined($xref)){
         $self->{_ecode} = ERECV;
         return undef;
@@ -807,11 +810,11 @@ sub sync {
         return 0;
     }
     my $sbuf = pack("CC", 0xC8, 0x70);
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -839,11 +842,11 @@ sub optimize {
     }
     my $sbuf = pack("CCN", 0xC8, 0x71, bytes::length($params));
     $sbuf .= $params;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -868,11 +871,11 @@ sub vanish {
         return 0;
     }
     my $sbuf = pack("CC", 0xC8, 0x72);
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -899,11 +902,11 @@ sub copy {
     }
     my $sbuf = pack("CCN", 0xC8, 0x73, bytes::length($path));
     $sbuf .= $path;
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -928,11 +931,11 @@ sub rnum {
         return 0;
     }
     my $sbuf = pack("CC", 0xC8, 0x80);
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -941,7 +944,7 @@ sub rnum {
         $self->{_ecode} = EMISC;
         return 0;
     }
-    my $rv = _recvint64($sock);
+    my $rv = $self->_recvint64();
     if($rv < 0){
         $self->{_ecode} = ERECV;
         return 0;
@@ -962,11 +965,11 @@ sub size {
         return 0;
     }
     my $sbuf = pack("CC", 0xC8, 0x81);
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return 0;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return 0;
@@ -975,7 +978,7 @@ sub size {
         $self->{_ecode} = EMISC;
         return 0;
     }
-    my $rv = _recvint64($sock);
+    my $rv = $self->_recvint64();
     if($rv < 0){
         $self->{_ecode} = ERECV;
         return 0;
@@ -996,11 +999,11 @@ sub stat {
         return undef;
     }
     my $sbuf = pack("CC", 0xC8, 0x88);
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return undef;
     }
-    my $code = _recvcode($sock);
+    my $code = $self->_recvcode();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return undef;
@@ -1009,12 +1012,12 @@ sub stat {
         $self->{_ecode} = EMISC;
         return undef;
     }
-    my $ssiz = _recvint32($sock);
+    my $ssiz = $self->_recvint32();
     if($ssiz < 0){
         $self->{_ecode} = ERECV;
         return undef;
     }
-    my $sref = _recv($sock, $ssiz);
+    my $sref = $self->_recv($ssiz);
     if(!defined($sref)){
         $self->{_ecode} = ERECV;
         return undef;
@@ -1045,12 +1048,12 @@ sub misc {
     foreach my $arg (@$args){
         $sbuf .= pack("N", bytes::length($arg)) . $arg;
     }
-    if(!_send($sock, \$sbuf)){
+    if(!$self->_send(\$sbuf)){
         $self->{_ecode} = ESEND;
         return undef;
     }
-    my $code = _recvcode($sock);
-    my $rnum = _recvint32($sock);
+    my $code = $self->_recvcode();
+    my $rnum = $self->_recvint32();
     if($code == -1){
         $self->{_ecode} = ERECV;
         return undef;
@@ -1061,12 +1064,12 @@ sub misc {
     }
     my @res;
     for(my $i = 0; $i < $rnum; $i++){
-        my $esiz = _recvint32($sock);
+        my $esiz = $self->_recvint32();
         if($esiz < 0){
             $self->{_ecode} = ERECV;
             return undef;
         }
-        my $eref = _recv($sock, $esiz);
+        my $eref = $self->_recv($esiz);
         if(!defined($eref)){
             $self->{_ecode} = ERECV;
             return undef;
@@ -1142,12 +1145,17 @@ sub CLEAR {
 
 
 sub _send {
-    my $sock = shift;
+    my $self = shift;
     my $ref = shift;
     my $len = bytes::length($$ref);
     while(1){
-        my $rv = send($sock, $$ref, 0);
-        return 1 if(!defined($rv));
+        if($self->{_tout} > 0){
+            my $bits = "";
+            vec($bits, fileno($self->{_sock}), 1) = 1;
+            return 0 if(select(undef, $bits, undef, $self->{_tout}) < 1);
+        }
+        my $rv = send($self->{_sock}, $$ref, 0);
+        return 0 if(!defined($rv));
         last if($rv == $len);
         my $str = substr($$ref, $rv);
         $len = bytes::length($str);
@@ -1158,18 +1166,28 @@ sub _send {
 
 
 sub _recv {
-    my $sock = shift;
+    my $self = shift;
     my $len = shift;
     my $str = "";
     return \$str if($len < 1);
-    return undef if(!defined(recv($sock, $str, $len, 0)));
+    if($self->{_tout} > 0){
+        my $bits = "";
+        vec($bits, fileno($self->{_sock}), 1) = 1;
+        return undef if(select($bits, undef, undef, $self->{_tout}) < 1);
+    }
+    return undef if(!defined(recv($self->{_sock}, $str, $len, 0)));
     return \$str if(bytes::length($str) == $len);
     $len -= bytes::length($str);
     while($len > 0){
         my $tstr = "";
-        return undef if(!defined(recv($sock, $tstr, $len, 0)));
+        if($self->{_tout} > 0){
+            my $bits = "";
+            vec($bits, fileno($self->{_sock}), 1) = 1;
+            return undef if(select($bits, undef, undef, $self->{_tout}) < 1);
+        }
+        return undef if(!defined(recv($self->{_sock}, $tstr, $len, 0)));
         if(bytes::length($tstr) < 1){
-            return undef if(!defined(recv($sock, $tstr, $len, 0)));
+            return undef if(!defined(recv($self->{_sock}, $tstr, $len, 0)));
             return undef if(bytes::length($tstr) < 1);
         }
         $len -= bytes::length($tstr);
@@ -1180,16 +1198,16 @@ sub _recv {
 
 
 sub _recvcode {
-    my $sock = shift;
-    my $rbuf = _recv($sock, 1);
+    my $self = shift;
+    my $rbuf = $self->_recv(1);
     return -1 if(!defined($rbuf));
     return unpack("C", $$rbuf);
 }
 
 
 sub _recvint32 {
-    my $sock = shift;
-    my $rbuf = _recv($sock, 4);
+    my $self = shift;
+    my $rbuf = $self->_recv(4);
     return 0 if(!defined($rbuf));
     my $num = unpack("N", $$rbuf);
     return unpack("l", pack("l", $num));
@@ -1197,8 +1215,8 @@ sub _recvint32 {
 
 
 sub _recvint64 {
-    my $sock = shift;
-    my $rbuf = _recv($sock, 8);
+    my $self = shift;
+    my $rbuf = $self->_recv(8);
     return -1 if(!defined($rbuf));
     my ($high, $low) = unpack("NN", $$rbuf);
     my $num = $high * _QUADBASE + $low;
