@@ -54,13 +54,13 @@ sub usage {
     printf STDERR ("$0: test cases of the remote database API\n");
     printf STDERR ("\n");
     printf STDERR ("usage:\n");
-    printf STDERR ("  $0 write [-port num] [-nr] [-rnd] host rnum\n");
-    printf STDERR ("  $0 read [-port num] [-mul num] [-rnd] host rnum\n");
-    printf STDERR ("  $0 remove [-port num] [-rnd] host rnum\n");
-    printf STDERR ("  $0 rcat [-port num] [-shl num] [-dai|-dad] [-ext name] [-xlg|-xlr]" .
-                   " host rnum\n");
-    printf STDERR ("  $0 misc [-port num] host rnum\n");
-    printf STDERR ("  $0 table [-port num] host rnum\n");
+    printf STDERR ("  $0 write [-port num] [-tout num] [-nr] [-rnd] host rnum\n");
+    printf STDERR ("  $0 read [-port num] [-tout num] [-mul num] [-rnd] host rnum\n");
+    printf STDERR ("  $0 remove [-port num] [-tout num] [-rnd] host rnum\n");
+    printf STDERR ("  $0 rcat [-port num] [-tout num] [-shl num] [-dai|-dad] [-ext name]" .
+                   " [-xlg|-xlr] host rnum\n");
+    printf STDERR ("  $0 misc [-port num] [-tout num] host rnum\n");
+    printf STDERR ("  $0 table [-port num] [-tout num] host rnum\n");
     printf STDERR ("\n");
     exit(1);
 }
@@ -71,6 +71,7 @@ sub runwrite {
     my $host = undef;
     my $rnum = undef;
     my $port = 1978;
+    my $tout = 0;
     my $nr = 0;
     my $rnd = 0;
     for(my $i = 1; $i < scalar(@ARGV); $i++){
@@ -78,6 +79,9 @@ sub runwrite {
             if($ARGV[$i] eq "-port"){
                 usage() if(++$i > scalar(@ARGV));
                 $port = TokyoTyrant::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-tout"){
+                usage() if(++$i > scalar(@ARGV));
+                $tout = TokyoTyrant::atoi($ARGV[$i]);
             } elsif($ARGV[$i] eq "-nr"){
                 $nr = 1;
             } elsif($ARGV[$i] eq "-rnd"){
@@ -94,7 +98,7 @@ sub runwrite {
         }
     }
     usage() if(!defined($host) || !defined($rnum) || $rnum < 1);
-    my $rv = procwrite($host, $port, $rnum, $nr, $rnd);
+    my $rv = procwrite($host, $port, $tout, $rnum, $nr, $rnd);
     return $rv;
 }
 
@@ -103,6 +107,7 @@ sub runwrite {
 sub runread {
     my $host = undef;
     my $port = 1978;
+    my $tout = 0;
     my $mul = 0;
     my $rnd = 0;
     for(my $i = 1; $i < scalar(@ARGV); $i++){
@@ -110,6 +115,9 @@ sub runread {
             if($ARGV[$i] eq "-port"){
                 usage() if(++$i > scalar(@ARGV));
                 $port = TokyoTyrant::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-tout"){
+                usage() if(++$i > scalar(@ARGV));
+                $tout = TokyoTyrant::atoi($ARGV[$i]);
             } elsif($ARGV[$i] eq "-mul"){
                 usage() if(++$i > scalar(@ARGV));
                 $mul = TokyoTyrant::atoi($ARGV[$i]);
@@ -125,7 +133,7 @@ sub runread {
         }
     }
     usage() if(!defined($host));
-    my $rv = procread($host, $port, $mul, $rnd);
+    my $rv = procread($host, $port, $tout, $mul, $rnd);
     return $rv;
 }
 
@@ -134,12 +142,16 @@ sub runread {
 sub runremove {
     my $host = undef;
     my $port = 1978;
+    my $tout = 0;
     my $rnd = 0;
     for(my $i = 1; $i < scalar(@ARGV); $i++){
         if(!defined($host) && $ARGV[$i] =~ /^-/){
             if($ARGV[$i] eq "-port"){
                 usage() if(++$i > scalar(@ARGV));
                 $port = TokyoTyrant::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-tout"){
+                usage() if(++$i > scalar(@ARGV));
+                $tout = TokyoTyrant::atoi($ARGV[$i]);
             } elsif($ARGV[$i] eq "-rnd"){
                 $rnd = 1;
             } else {
@@ -152,7 +164,7 @@ sub runremove {
         }
     }
     usage() if(!defined($host));
-    my $rv = procremove($host, $port, $rnd);
+    my $rv = procremove($host, $port, $tout, $rnd);
     return $rv;
 }
 
@@ -162,6 +174,7 @@ sub runrcat {
     my $host = undef;
     my $rnum = undef;
     my $port = 1978;
+    my $tout = 0;
     my $shl = 0;
     my $dai = 0;
     my $dad = 0;
@@ -172,6 +185,9 @@ sub runrcat {
             if($ARGV[$i] eq "-port"){
                 usage() if(++$i > scalar(@ARGV));
                 $port = TokyoTyrant::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-tout"){
+                usage() if(++$i > scalar(@ARGV));
+                $tout = TokyoTyrant::atoi($ARGV[$i]);
             } elsif($ARGV[$i] eq "-shl"){
                 usage() if(++$i > scalar(@ARGV));
                 $shl = TokyoTyrant::atoi($ARGV[$i]);
@@ -198,7 +214,7 @@ sub runrcat {
         }
     }
     usage() if(!defined($host) || !defined($rnum) || $rnum < 1);
-    my $rv = procrcat($host, $port, $rnum, $shl, $dai, $dad, $ext, $xopts);
+    my $rv = procrcat($host, $port, $tout, $rnum, $shl, $dai, $dad, $ext, $xopts);
     return $rv;
 }
 
@@ -208,11 +224,15 @@ sub runmisc {
     my $host = undef;
     my $rnum = undef;
     my $port = 1978;
+    my $tout = 0;
     for(my $i = 1; $i < scalar(@ARGV); $i++){
         if(!defined($host) && $ARGV[$i] =~ /^-/){
             if($ARGV[$i] eq "-port"){
                 usage() if(++$i > scalar(@ARGV));
                 $port = TokyoTyrant::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-tout"){
+                usage() if(++$i > scalar(@ARGV));
+                $tout = TokyoTyrant::atoi($ARGV[$i]);
             } else {
                 usage();
             }
@@ -225,7 +245,7 @@ sub runmisc {
         }
     }
     usage() if(!defined($host) || !defined($rnum) || $rnum < 1);
-    my $rv = procmisc($host, $port, $rnum);
+    my $rv = procmisc($host, $port, $tout, $rnum);
     return $rv;
 }
 
@@ -235,11 +255,15 @@ sub runtable {
     my $host = undef;
     my $rnum = undef;
     my $port = 1978;
+    my $tout = 0;
     for(my $i = 1; $i < scalar(@ARGV); $i++){
         if(!defined($host) && $ARGV[$i] =~ /^-/){
             if($ARGV[$i] eq "-port"){
                 usage() if(++$i > scalar(@ARGV));
                 $port = TokyoTyrant::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-tout"){
+                usage() if(++$i > scalar(@ARGV));
+                $tout = TokyoTyrant::atoi($ARGV[$i]);
             } else {
                 usage();
             }
@@ -252,7 +276,7 @@ sub runtable {
         }
     }
     usage() if(!defined($host) || !defined($rnum) || $rnum < 1);
-    my $rv = proctable($host, $port, $rnum);
+    my $rv = proctable($host, $port, $tout, $rnum);
     return $rv;
 }
 
@@ -270,15 +294,16 @@ sub eprint {
 sub procwrite {
     my $host = shift;
     my $port = shift;
+    my $tout = shift;
     my $rnum = shift;
     my $nr = shift;
     my $rnd = shift;
-    printf("<Writing Test>\n  host=%s  port=%d  rnum=%d  nr=%d  rnd=%d\n\n",
-           $host, $port, $rnum, $nr, $rnd);
+    printf("<Writing Test>\n  host=%s  port=%d  tout=%d  rnum=%d  nr=%d  rnd=%d\n\n",
+           $host, $port, $tout, $rnum, $nr, $rnd);
     my $err = 0;
     my $stime = gettimeofday();
     my $rdb = TokyoTyrant::RDB->new();
-    if(!$rdb->open($host, $port)){
+    if(!$rdb->open($host, $port, $tout)){
         eprint($rdb, "open");
         $err = 1;
     }
@@ -324,13 +349,15 @@ sub procwrite {
 sub procread {
     my $host = shift;
     my $port = shift;
+    my $tout = shift;
     my $mul = shift;
     my $rnd = shift;
-    printf("<Reading Test>\n  host=%s  port=%d  mul=%d  rnd=%d\n\n", $host, $port, $mul, $rnd);
+    printf("<Reading Test>\n  host=%s  port=%d  tout=%d  mul=%d  rnd=%d\n\n",
+           $host, $port, $tout, $mul, $rnd);
     my $err = 0;
     my $stime = gettimeofday();
     my $rdb = TokyoTyrant::RDB->new();
-    if(!$rdb->open($host, $port)){
+    if(!$rdb->open($host, $port, $tout)){
         eprint($rdb, "open");
         $err = 1;
     }
@@ -378,12 +405,14 @@ sub procread {
 sub procremove {
     my $host = shift;
     my $port = shift;
+    my $tout = shift;
     my $rnd = shift;
-    printf("<Removing Test>\n  host=%s  port=%d  rnd=%d\n\n", $host, $port, $rnd);
+    printf("<Removing Test>\n  host=%s  port=%d  tout=%d  rnd=%d\n\n",
+           $host, $port, $tout, $rnd);
     my $err = 0;
     my $stime = gettimeofday();
     my $rdb = TokyoTyrant::RDB->new();
-    if(!$rdb->open($host, $port)){
+    if(!$rdb->open($host, $port, $tout)){
         eprint($rdb, "open");
         $err = 1;
     }
@@ -418,20 +447,21 @@ sub procremove {
 sub procrcat {
     my $host = shift;
     my $port = shift;
+    my $tout = shift;
     my $rnum = shift;
     my $shl = shift;
     my $dai = shift;
     my $dad = shift;
     my $ext = shift;
     my $xopts = shift;
-    printf("<Random Concatenating Test>\n  host=%s  port=%d  rnum=%d  shl=%d  dai=%d  dad=%d" .
-           "  ext=%s  xopts=%d\n\n",
-           $host, $port, $rnum, $shl, $dai, $dad, $ext ? $ext : "", $xopts);
+    printf("<Random Concatenating Test>\n  host=%s  port=%d  tout=%d  rnum=%d" .
+           "  shl=%d  dai=%d  dad=%d  ext=%s  xopts=%d\n\n",
+           $host, $port, $tout, $rnum, $shl, $dai, $dad, $ext ? $ext : "", $xopts);
     my $pnum = $rnum / 5 + 1;
     my $err = 0;
     my $stime = gettimeofday();
     my $rdb = TokyoTyrant::RDB->new();
-    if(!$rdb->open($host, $port)){
+    if(!$rdb->open($host, $port, $tout)){
         eprint($rdb, "open");
         $err = 1;
     }
@@ -495,12 +525,14 @@ sub procrcat {
 sub procmisc {
     my $host = shift;
     my $port = shift;
+    my $tout = shift;
     my $rnum = shift;
-    printf("<Miscellaneous Test>\n  host=%s  port=%d  rnum=%d\n\n", $host, $port, $rnum);
+    printf("<Miscellaneous Test>\n  host=%s  port=%d  tout=%d  rnum=%d\n\n",
+           $host, $port, $tout, $rnum);
     my $err = 0;
     my $stime = gettimeofday();
     my $rdb = TokyoTyrant::RDB->new();
-    if(!$rdb->open($host, $port)){
+    if(!$rdb->open($host, $port, $tout)){
         eprint($rdb, "open");
         $err = 1;
     }
@@ -782,12 +814,14 @@ sub procmisc {
 sub proctable {
     my $host = shift;
     my $port = shift;
+    my $tout = shift;
     my $rnum = shift;
-    printf("<Table Extension Test>\n  host=%s  port=%d  rnum=%d\n\n", $host, $port, $rnum);
+    printf("<Table Extension Test>\n  host=%s  port=%d  tout=%d  rnum=%d\n\n",
+           $host, $port, $tout, $rnum);
     my $err = 0;
     my $stime = gettimeofday();
     my $rdb = TokyoTyrant::RDBTBL->new();
-    if(!$rdb->open($host, $port)){
+    if(!$rdb->open($host, $port, $tout)){
         eprint($rdb, "open");
         $err = 1;
     }
@@ -835,10 +869,10 @@ sub proctable {
             $vbuf .= "," if(length($vbuf) > 0);
             $vbuf .= $pt;
         }
-	if(length($vbuf) > 0){
-	    $cols->{flag} = $vbuf;
-	    $cols->{text} = $vbuf;
-	}
+        if(length($vbuf) > 0){
+            $cols->{flag} = $vbuf;
+            $cols->{text} = $vbuf;
+        }
         if(!$rdb->put($id, $cols)){
             eprint($rdb, "put");
             $err = 1;
